package saferepo.ui.javafx;

import java.io.File;
import java.nio.file.Paths;

import javafx.collections.ObservableList;
import javafx.scene.control.Button;
import javafx.scene.control.Label;
import javafx.scene.control.ListView;
import javafx.scene.layout.BorderPane;
import javafx.scene.layout.HBox;
import javafx.scene.layout.VBox;
import javafx.stage.DirectoryChooser;
import javafx.stage.FileChooser;
import javafx.stage.Stage;
import saferepo.model.VersionedDocument;
import saferepo.persistence.SadPersisterException;
import saferepo.ui.controller.Controller;

public class SafeRepoPane extends BorderPane {

	private Controller controller;
	private Stage stage;
	private VBox vbox1, vbox2;
	private ListView<String> listaDocumenti;
	private ListView<VersionedDocument> listaVersioni;
	private Button aggiungi, elimina;
	private String selectedDocumentID;
	@SuppressWarnings("unused")
	private VersionedDocument selectedVersion;
	
	public SafeRepoPane(Controller controller, Stage stage) {
		this.controller = controller;
		this.stage=stage;
		//
		listaDocumenti = new ListView<>(controller.getDocumentIDs());
		listaDocumenti.setPrefHeight(150);
		listaDocumenti.setPrefWidth(150);
		vbox1 = new VBox();
		vbox1.getChildren().add(new Label(" Elenco documenti: "));
		vbox1.getChildren().add(listaDocumenti);
		listaDocumenti.getSelectionModel().selectedItemProperty().addListener(
				(obj, oldval, newval) -> { docSelHandler(newval); selectedDocumentID = newval; }
		);
		//
		listaVersioni = new ListView<>();
		listaVersioni.setPrefHeight(150);
		listaVersioni.setPrefWidth(350);
		vbox2 = new VBox();
		vbox2.getChildren().add(new Label(" Elenco versioni: "));
		vbox2.getChildren().add(listaVersioni);
		listaVersioni .getSelectionModel().selectedItemProperty().addListener(
				(obj, oldval, newval) -> { retrieveDocVersion(newval); selectedVersion = newval; }
		);
		//
		HBox topPane = new HBox();
		topPane.getChildren().addAll(vbox1, vbox2);
		this.setTop(topPane);
		//
		HBox centerPane = new HBox();
		aggiungi = new Button("Aggiungi file");
		aggiungi.setOnAction(e -> addDocument());
		elimina = new Button("Elimina file");
		elimina.setOnAction(e -> delDocument());
		elimina.setDisable(true);
		centerPane.getChildren().addAll(aggiungi, elimina);
		this.setCenter(centerPane);
	}

	private void docSelHandler(String documentID) {
		 if (documentID==null) return; // per eliminare effetti spuri dovuti a inserzione nuovi documenti che causano aggiornamenti interni alla lista, senza che ci sia un documento selezionato
		 // System.out.println("Selezionato file: " + documentID);
		 ObservableList<VersionedDocument> versioniDocumentID = controller.getVersions(documentID);
		 // System.out.println("Popolata lista versioni con " + versioniDocumentID);
		 listaVersioni.setItems(versioniDocumentID);
		 elimina.setDisable( versioniDocumentID.size()==0 );
	}
	
	public void retrieveDocVersion(VersionedDocument vdoc) {
		// System.out.println("Selezionato file: " + vdoc.getID());
		if (vdoc.getDocument()==null) {
			Controller.alert("Errore: file cancellato", "Il file " + vdoc.toString() + "è stato eliminato", "(sono comunque disponibili le versioni precedenti)");
			return;
		}

		DirectoryChooser chooser = new DirectoryChooser();
		chooser.setTitle("Recupero versione #" + vdoc.getVersion() + " di " + vdoc.getID());
		chooser.setInitialDirectory(Paths.get(System.getProperty("user.dir"), "retrievedFiles").toFile());
		File selectedFile = chooser.showDialog(stage);
		if (selectedFile==null) return; // utente ha annullato operazione
		try {
			VersionedDocument savedDoc = controller.retrieve(vdoc, selectedFile.toPath());
			Controller.info("Scaricamento completato", "Il documento è disponibile in " + savedDoc.getPath().toString(), "Saluti!");
		} catch (SadPersisterException e) {
			Controller.alert("Errore irrecuperabile", "Errore nel recupero di " + vdoc.toString(), "Peccato..!");
		}
	}

	private void addDocument() {
		FileChooser chooser = new FileChooser();
		chooser.setTitle("Aggiunta documento");
		chooser.setInitialDirectory(Paths.get(System.getProperty("user.dir"), "testFiles").toFile());
		File selectedFile = chooser.showOpenDialog(stage);
		if (selectedFile==null) return; // utente ha annullato operazione
		try {
			@SuppressWarnings("unused")
			VersionedDocument vdoc = controller.store(selectedFile.toPath().getFileName().toString(), selectedFile.toPath());
			listaDocumenti.setItems(controller.getDocumentIDs());
		} catch (SadPersisterException e) {
			Controller.alert("Errore irrecuperabile", "Errore nell'aggiunta di " + selectedFile, "Che peccato!");
		}
	}

	private void delDocument() {
			boolean confirmed = Controller.askConfirm("Richiesta eliminazione", "Eliminare davvero il documento " + selectedDocumentID + "?", "Pensaci bene..");
			if (confirmed) {
				try {
					controller.delete(selectedDocumentID);
					listaDocumenti.setItems(controller.getDocumentIDs());
				} catch (SadPersisterException e) {
					Controller.alert("Errore irrecuperabile", "Errore nell'eliminazione di " + selectedDocumentID, "Situazione inconsistente");
				}
			}
	}

}
